from __future__ import annotations

from .core.events import DatasetEvent, FitEvent, PingEvent, PredictEvent
from .core.service import ProductTelemetry, capture_event
from .core.decorators import (
    track_model_call,
    set_extension,
    get_current_extension,
    set_model_config,
)

# Public exports
__all__ = [
    "DatasetEvent",
    "FitEvent",
    "PingEvent",
    "PredictEvent",
    "ProductTelemetry",
    "capture_event",
    "track_model_call",
    "set_extension",
    "get_current_extension",
    "set_model_config",
]
